#include "186COMM.h"

char CmdMsg[512]; //For storing the data of received command
unsigned int CmdLen; //For store the data length of recevie command
unsigned int SendMsgFlag,ErrorFlag;
void UserDPRAMIrqFunc(unsigned char INTT) //must be called
{
	int Ret;
	
	if (INTT==SEND_DPRAM_CMD){ //Check if commmand is received by using interrupt service routine
		Ret=DPRAMReceiveCmd(CmdMsg,&CmdLen);
		if (!Ret){
			SendMsgFlag=1; //Trigger the flag of sending message, then send message in loop function
		}
		else{
			if (Ret!=_NO_DPRAM_CMD) ErrorFlag=Ret; //Trigger the error flag, then show the error message in loop function
		}
	}
}
void UserCANIrqFunc(unsigned char INTT)   //must be called
{
  //Do nothing
}
void UserInitFunc(void)    //must be called
{
	int Ret;
  Print("TransmitMsg demo is runnung...\r\n");
  DebugPrint("TransmitMsg demo is runnung...\r\n");
  SendMsgFlag=ErrorFlag=0; //initial the golbal variable
  Ret=CANConfig(125000UL,0,0,0xffffffffUL,0x00000000UL); //Set baud to 125K, and filter is "passing all messages"
  if (Ret){
  	Print("Config error, error code=%d\r\n",Ret);
  	DebugPrint("Config error, error code=%d\r\n",Ret);
  }
  EnableSJA1000(); //Let SJA1000 start to work.
}
void UserLoopFunc(void)    //must be called
{
	char Data[8],TempData[3],DataLen,Mode,RTR,i,ReadyToSend=0,*endptr;
	int Ret;
	unsigned long MsgID,LowMsgID;
	
	if (SendMsgFlag){ //Check if the flag of sending message is triggered.
		switch(CmdMsg[0]){ //Check command type
			case 'T': //send 11-bit ID message with RTR, command format is IIIL (III means the hax value of ID, L means datalen)
				DebugPrint("CmdLen=%d\r\n",CmdLen);
				if (CmdLen>4){//Check CmdLen is OK or not
					Mode=0;
					RTR=1;  
					DataLen=(CmdMsg[4]-0x30); //Get datalen (from ascii to value)
					if (DataLen>8) DataLen=8;
					CmdMsg[4]=0;
					MsgID=strtoul(&CmdMsg[1],&endptr,16); //Get message ID (from ascii to value)
					ReadyToSend=1;
				}
				break;
			case 't': //send 11-bit ID message, command format is IIILDDDDDD...(DD means a byte of data with hex format)
				if (CmdLen>4){//Check CmdLen is OK or not
					Mode=0;
					RTR=0;
					DataLen=(CmdMsg[4]-0x30);
					if (DataLen>8) DataLen=8; //Get datalen (from ascii to value)
					if (CmdLen<=(4+DataLen*2)) break; //CmdLen is not enough for message data
					CmdMsg[4]=0;
					MsgID=strtoul(&CmdMsg[1],&endptr,16); //Get message ID (from ascii to value)
					TempData[2]=0;
					for (i=0;i<DataLen;i++){ //Get data (from ascii to value)
						TempData[0]=CmdMsg[5+i*2];
						TempData[1]=CmdMsg[5+i*2+1];
						Data[i]=(char)strtoul(TempData,&endptr,16);
					}
					ReadyToSend=1;
				}
				break;
			case 'E': //send 29-bit ID message with RTR, command format is IIIIIIIIL.
				if (CmdLen>9){//Check CmdLen is OK or not
					Mode=1;
					RTR=1;
					DataLen=(CmdMsg[9]-0x30); //Get datalen (from ascii to value)
					if (DataLen>8) DataLen=8;
					CmdMsg[9]=0;
					LowMsgID=strtoul(&CmdMsg[5],&endptr,16); //Get message ID (from ascii to value)
					CmdMsg[5]=0;
					MsgID=strtoul(&CmdMsg[1],&endptr,16); //avoid the bug of strtoul() of TC++ 1.01
					MsgID=(MsgID<<16)+LowMsgID;
					ReadyToSend=1;
				}
				break;			
			case 'e': //send 29-bit ID message, command format is IIIIIIIILDDDDDD...
				if (CmdLen>9){//Check CmdLen is OK or not
					Mode=1;
					RTR=0;
					DataLen=(CmdMsg[9]-0x30); //Get datalen (from ascii to value)
					if (DataLen>8) DataLen=8;
					if (CmdLen<=(9+DataLen*2)) break; //CmdLen is not enough for message data
					CmdMsg[9]=0;
					LowMsgID=strtoul(&CmdMsg[5],&endptr,16); //Get message ID (from ascii to value)
					CmdMsg[5]=0;
					MsgID=strtoul(&CmdMsg[1],&endptr,16); //avoid the bug of strtoul() of TC++ 1.01
					MsgID=(MsgID<<16)+LowMsgID;
					TempData[2]=0;
					for (i=0;i<DataLen;i++){ //Get data (from ascii to value)
						TempData[0]=CmdMsg[10+i*2];
						TempData[1]=CmdMsg[10+i*2+1];
						Data[i]=(char)strtoul(TempData,&endptr,16);
					}
					ReadyToSend=1;
				}
				break;
		}
		RefreshWDT();// avoid the WDT timeout
		if (ReadyToSend){ //Check if need to send message or not
			Ret=SendCANMsg(Mode,MsgID,RTR,DataLen,Data);
			if (Ret){
				Print("SendCANMsg error, error code=%d\r\n",Ret);
  		  DebugPrint("SendCANMsg error, error code=%d\r\n",Ret);
			}
			else{
				Print("Send one CAN message\r\n");
  		  DebugPrint("Send one CAN message\r\n");
		  }
			ReadyToSend=0;
		}
		SendMsgFlag=0;
	}
	if (ErrorFlag){
		Print("DPRAMReceiveCmd error, error code=%d\r\n",ErrorFlag);
  	DebugPrint("DPRAMReceiveCmd error, error code=%d\r\n",ErrorFlag);
  	ErrorFlag=0;
	}
}